precision highp float;

uniform sampler2D inputImageTexture;

uniform vec2 singleStepOffset; 
uniform highp vec4 params; 

varying highp vec2 textureCoordinate;

const highp vec3 W = vec3(0.299,0.587,0.114);  //YUV 转换


const mat3 saturateMatrix = mat3(
		1.1102,-0.0598,-0.061,
		-0.0774,1.0826,-0.1186,
		-0.0228,-0.0228,1.1772);
		
float hardlight(float color)
{
	if(color <= 0.5)
	{
		color = color * color * 2.0;
	}
	else
	{
		color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
	}
	return color;
}

bool isSkin(vec3 color) {
    if ((color.r > 0.37109 && color.g > 0.15625 && color.b > 0.078125
                && color.r - color.b > 0.05859 && color.r - color.g > 0.05859)
                ||
                (color.r > 0.78125 && color.g > 0.8203125 && color.b > 0.6640625
                 && abs(color.r - color.b) <= 0.05859 && color.r > color.b
                 && color.g > color.b )) {
        return true;
    } else {
        return false;
    }
}


float getblurSkinColorY(float centerColorY, vec2 point) {
    vec3 thisColor = texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * point).rgb;
    if(isSkin(thisColor)) {
        return dot(thisColor, W);
    } else {
        return centerColorY;
    }
}

//局部方差磨皮，保留细节

void main(){

    vec3 centralColor = texture2D(inputImageTexture, textureCoordinate).rgb;
    float centralColorY = dot(centralColor, W);
//    if (isSkin(centralColor)) {

        float blurColor[49];
        int index = 0;
        float mm = 0.0;

        for(int i = -3; i <= 3; i++) {
            for(int j = -3; j <= 3; j++) {
                blurColor[index] = dot(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(float(i), float(j))).rgb, W);
                mm += blurColor[index];
                index++;
            }
        }
        mm = mm / 49.0;

        index = 0;
        float v = 0.0;
        for(int i = -3; i <= 3; i++) {
            for(int j = -3; j <= 3; j++) {
                v += pow(blurColor[index] - mm, 2.0);
                index++;
            }
        }
        v = v / 49.0;

        float k = v / (v + params.r);

        float lumance = dot(centralColor, W);   //yuv 转换  Y 明度

        float yy = (1.0 - k) * mm + k * lumance;
        float cr = 0.500 * centralColor.r - 0.4187 * centralColor.g - 0.0813 * centralColor.b + 0.5;
        float cb = -0.1687 * centralColor.r - 0.3313 * centralColor.g + 0.500 * centralColor.b + 0.5;



        vec3 smoothColor = vec3(
            yy + 1.4075 * (cr - 0.5),
            yy - 0.3455 * (cb - 0.5) - 0.7169 * (cr - 0.5),
            yy + 1.779 * (cb - 0.5)
        );
        gl_FragColor = vec4(smoothColor, 1.0);
//    } else {
//        gl_FragColor = vec4(centralColor, 1.0);
//    }


//	gl_FragColor = vec4(mix(smoothColor.rgb, max(smoothColor, centralColor), alpha), 1.0);

//	float highpass = centralColor.g - sampleColor + 0.5;  //高通？
//
//	for(int i = 0; i < 5;i++)
//	{
//		highpass = hardlight(highpass);
//	}
//	float lumance = dot(centralColor, W);   //yuv 转换  Y 明度
//
//	float alpha = pow(lumance, params.r);
//
//	vec3 smoothColor = centralColor + (centralColor-vec3(highpass))*alpha*0.1;
//
//	smoothColor.r = clamp(pow(smoothColor.r, params.g),0.0,1.0);
//	smoothColor.g = clamp(pow(smoothColor.g, params.g),0.0,1.0);
//	smoothColor.b = clamp(pow(smoothColor.b, params.g),0.0,1.0);
	
//	vec3 lvse = vec3(1.0)-(vec3(1.0)-smoothColor)*(vec3(1.0)-centralColor);
//	vec3 bianliang = max(smoothColor, centralColor);
//	vec3 rouguang = 2.0*centralColor*smoothColor + centralColor*centralColor - 2.0*centralColor*centralColor*smoothColor;
//
//	gl_FragColor = vec4(mix(centralColor, lvse, alpha), 1.0);
//	gl_FragColor.rgb = mix(gl_FragColor.rgb, bianliang, alpha);
//	gl_FragColor.rgb = mix(gl_FragColor.rgb, rouguang, params.b);
//
//	vec3 satcolor = gl_FragColor.rgb * saturateMatrix;
//	gl_FragColor.rgb = mix(gl_FragColor.rgb, satcolor, params.a);
}