precision highp float;

uniform sampler2D inputImageTexture;
uniform sampler2D skinTexture;

uniform vec2 singleStepOffset; 

varying highp vec2 textureCoordinate;

const highp vec3 W = vec3(0.299,0.587,0.114);  //YUV 转换

float getSkinAlpha() {
      float skinColor = texture2D(skinTexture, textureCoordinate.xy + singleStepOffset * vec2(-1, 0)).r
                   + texture2D(skinTexture, textureCoordinate.xy + singleStepOffset * vec2(0, -1)).r
                   + texture2D(skinTexture, textureCoordinate.xy + singleStepOffset * vec2(0, 0)).r
                   + texture2D(skinTexture, textureCoordinate.xy + singleStepOffset * vec2(0, 1)).r
                   + texture2D(skinTexture, textureCoordinate.xy + singleStepOffset * vec2(1, 0)).r;
    return (skinColor / 5.0);
}


void main(){
    vec3 centralColor = texture2D(inputImageTexture, textureCoordinate).rgb;
    float centralColorY = dot(centralColor, W);
    float skinAlpha = getSkinAlpha();  //羽化边缘
    if (skinAlpha > 0.002) {
        float yy =  centralColorY * 8.0
                  - dot(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, -1.0)).rgb, W)
                  - dot(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, 1.0)).rgb, W)
                  - dot(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, -1.0)).rgb, W)
                  - dot(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, 1.0)).rgb, W);
        yy = yy / 4.0;

        yy = mix(centralColorY, yy, skinAlpha);;
        float cr = 0.500 * centralColor.r - 0.4187 * centralColor.g - 0.0813 * centralColor.b + 0.5;
        float cb = -0.1687 * centralColor.r - 0.3313 * centralColor.g + 0.500 * centralColor.b + 0.5;
        vec3 smoothColor = vec3(
            yy + 1.4075 * (cr - 0.5),
            yy - 0.3455 * (cb - 0.5) - 0.7169 * (cr - 0.5),
            yy + 1.779 * (cb - 0.5)
        );
        gl_FragColor = vec4(smoothColor, 1.0);
    } else {
        gl_FragColor = vec4(centralColor, 1.0);
    }

}