precision highp float;

uniform sampler2D inputImageTexture;

uniform vec2 singleStepOffset;
uniform vec2 retineSize;

varying highp vec2 textureCoordinate;

const highp vec3 W = vec3(0.299,0.587,0.114);  //YUV 转换

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

float rgb2v(vec3 c) {
    return c.r > c.g ? (c.r > c.b ? c.r : c.b) : (c.g > c.b ? c.g : c.b);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

float hardlight(float color)
{
	if(color <= 0.5)
	{
		color = 1.4142135623730951 * pow(color, 1.5); // color * color * 2.0;
	}
	else
	{
		color = 1.0 - pow((1.0 - color), 1.5) * 1.4142135623730951; //1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
	}
	return color;
}

void main(){
    float gauessV = rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, -3.0)).rgb) * 0.02039796
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, -2.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, -1.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, 0.0)).rgb) * 0.02040714
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, 1.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, 2.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-3.0, 3.0)).rgb) * 0.02039796
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, -3.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, -2.0)).rgb) * 0.02040816
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, -1.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, 0.0)).rgb) * 0.02041224
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, 1.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, 2.0)).rgb) * 0.02040816
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-2.0, 3.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, -3.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, -2.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, -1.0)).rgb) * 0.02041429
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, 0.0)).rgb) * 0.02041531
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, 1.0)).rgb) * 0.02041429
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, 2.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(-1.0, 3.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, -3.0)).rgb) * 0.02040714
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, -2.0)).rgb) * 0.02041224
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, -1.0)).rgb) * 0.02041531
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, 0.0)).rgb) * 0.02041633
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, 1.0)).rgb) * 0.02041531
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, 2.0)).rgb) * 0.02041224
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(0.0, 3.0)).rgb) * 0.02040714
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, -3.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, -2.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, -1.0)).rgb) * 0.02041429
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, 0.0)).rgb) * 0.02041531
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, 1.0)).rgb) * 0.02041429
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, 2.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(1.0, 3.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, -3.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, -2.0)).rgb) * 0.02040816
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, -1.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, 0.0)).rgb) * 0.02041224
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, 1.0)).rgb) * 0.02041122
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, 2.0)).rgb) * 0.02040816
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(2.0, 3.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, -3.0)).rgb) * 0.02039796
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, -2.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, -1.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, 0.0)).rgb) * 0.02040714
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, 1.0)).rgb) * 0.02040612
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, 2.0)).rgb) * 0.02040306
                + rgb2v(texture2D(inputImageTexture, textureCoordinate.xy + singleStepOffset * vec2(3.0, 3.0)).rgb) * 0.02039796
                ;

    vec3 sourcehsv = rgb2hsv(texture2D(inputImageTexture, textureCoordinate.xy).rgb);
    float retinexV = log(sourcehsv.b * 255.0 + 1.0) / log(10.0) - log(gauessV * 255.0 + 1.0) / log(10.0);
    retinexV = max(retineSize.x, retinexV);
    retinexV = min(retineSize.y, retinexV);
    float retinexRef = (retinexV - retineSize.x) / (retineSize.y - retineSize.x) + 1.0 / 255.0;
    float A = min(256.0 / (sourcehsv.b * 255.0 + 1.0) , retinexRef / (sourcehsv.b  + 1.0 / 255.0));
    float alpha = 0.4;
    float retinexRet = alpha * A * (sourcehsv.b  + 1.0 / 255.0) + (1.0 - alpha) * (sourcehsv.b  + 1.0 / 255.0) - 1.0 / 255.0;
    sourcehsv.b = hardlight(retinexRet);
    gl_FragColor = vec4(hsv2rgb(sourcehsv), 1.0);
//    float A = min(1.0 / sourcehsv.b, retinexRef / sourcehsv.b);
//    float alpha = 0.4;
//    sourcehsv.b = alpha * A * sourcehsv.b + (1.0 - alpha) * sourcehsv.b - 1.0 / 255.0;
//    gl_FragColor = vec4(hsv2rgb(sourcehsv), 1.0);
}